//
// osm.h
//
// Version 1.0
//
// Header for the Open Street Map (OSM) classes.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

#ifndef OSM_OSM_H
#define OSM_OSM_H

//
// Classes are defined for Node, Way and Relation.  Each have unique fields and
// a list of tags which can be zero length.  The classes are simple with public 
// fields and only Set or Get functions that add value. 
//
// Tags are stored in a multimap collection.  Ideally this would be a map 
// collection, but tags (in api 0.5) are not unique.
//

class OSMNode
// 
// A class to describe an OSM node.  A node is a single point with a position.  It
// can describe a single item, or be used as part of a way, or both.  A single node 
// needs one or more tags to describe it.  Nodes in a way often do not have tags, but
// they can have tags.
//
{
public:
	int NodeID;
		// The node id is a globally unique id.  If the id has not been allocated by 
		// the api server (for a new node) it will be temporarily negative.
	float lat;
		// The latitude of the node ddd.ddddddd
	float lon;
		// The longitude of the node ddd.ddddddd
	std::multimap<std::string, std::string> tags;
		// The list of tags attached to the node. This is multimap because keys
		// are not guaranteed to be unique (api 0.5).
	
	bool operator < (OSMNode);
		// The less than operator.  It compares the NodeID only.
	OSMNode();
		// Create the OSMNode.
	OSMNode(int);
		// Create the OSMNode with a specified Node ID.
	~OSMNode();
		// Destroy the OSMNode.  The list of tags is destroyed.
	void AddTag(std::string, std::string);
		// Add a tag with the key and value to the list of tags.
	void DisplayNode(std::ostream&);
		// Display the contents of the node - mostly for debugging.
};

class OSMWay
//
// A class to describe an OSM way.  A way is a collection of nodes.  The nodes
// will all be in the NodeList.  It will have one or more tags to describe it.
//
{
public:
	int WayID;
		// The way id is a globally unique id.  If the id has not been allocated by 
		// the api server (for a new way) it will be temporarily negative.
	std::multimap<std::string, std::string> tags;
		// The list of tags attached to the way. This is multimap because keys
		// are not guaranteed to be unique (api 0.5).
	std::list<int> nodes;
		// The list of NodeIDs that make up the way.  The sequence is significant.
	
	OSMWay();
		// Create the OSMWay.
	OSMWay(int);
		// Create the OSMWay with a specified ID.
	~OSMWay();
		// Destroy the OSMWay.  The lists of tags and nodes are destroyed.
	
	void AddNode(int);
		// Add a node id to the list of nodes.
	void AddTag(std::string, std::string);
		// Add a tag with the key and value to the list of tags.
	void DisplayWay(std::ostream&);
		// Display the contents of the way - mostly for debugging.
	
};

class OSMRelMember
//
// A class to describe an OSM relation member.  It is used in a collection
// in the OSM relation.
//
{
public:
	std::string type;
		// The member type (node, way or relation).
	int ref;
		// The Id of the member (node id, way id, relation id).
	std::string role;
		// The role of the member. Use depends on the relation but this is sometimes "".
	
	OSMRelMember();
	OSMRelMember(std::string, int, std::string);

	void DisplayMember(std::ostream&);
		// Display the contents of the member - mostly for debugging.
};

class OSMRelation
//
// A class to describe an OSM relation.  A relation is a collection other OSM
// objects which can be nodes, ways or relations.  Objects are stored in the 
// list of members.  The nodes, ways or relations are *NOT* guaranteed to be 
// included in the other lists. Relations have one or more tags to describe it.
//
{
public:
	int RelID;
		// The relation id is a globally unique id.  If the id has not been allocated 
		// by the api server (for a new relation) it will be temporarily negative.
	std::multimap<std::string, std::string> tags;
		// The list of tags attached to the relation. This is multimap because keys
		// are not guaranteed to be unique (api 0.5).
	std::list<OSMRelMember> members;
		// The list of members of the relation.
	
	OSMRelation();
		// Create the OSM relation.
	OSMRelation(int);
		// Create the OSM relation with a relation id.
	~OSMRelation();
		// Destroy the relation. The lists of tags and members are destroyed.
	
	void AddTag(std::string, std::string);
		// Add a tag with the key and value to the list of tags.
	void AddMember(OSMRelMember);
		// Add an OSMRelMember to the list of members.
	void DisplayRelation(std::ostream&);
		// Display the contents of the member - mostly for debugging.
};

#endif
